/* -*- c++ -*- */
/*
 * Copyright 2004 Free Software Foundation, Inc.
 * 
 * This file is part of GNU Radio
 * 
 * GNU Radio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * GNU Radio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GNU Radio; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*	HSR - MOBKOM LABOR
 *	Semesterarbeit GnuRadio Contributions
 *	U. Schaufelberger and R. Gaensli
 *
 * 	Written by : WRITTEN_BY
 * 	Created: CREATED_DATE
 * 
 */


#ifndef INCLUDED_gr_rds_data_decoder_H
#define INCLUDED_gr_rds_data_decoder_H

#include <gr_sync_block.h>
#include <vector>
#include <string>

class gr_rds_data_decoder;

/*
 * We use boost::shared_ptr's instead of raw pointers for all access
 * to gr_blocks (and many other data structures).  The shared_ptr gets
 * us transparent reference counting, which greatly simplifies storage
 * management issues.  This is especially helpful in our hybrid
 * 
 * * C++ / Python system.
 *
 * See http://www.boost.org/libs/smart_ptr/smart_ptr.htm
 *
 * As a convention, the _sptr suffix indicates a boost::shared_ptr
 */
typedef boost::shared_ptr<gr_rds_data_decoder> gr_rds_data_decoder_sptr;

/*!
 * \brief Return a shared_ptr to a new instance of gr_rds_data_decoder.
 *
 * To avoid accidental use of raw pointers, gr_rds_data_decoder's
 * constructor is private. make_square_ff is the public
 * interface for creating new instances.
 */
gr_rds_data_decoder_sptr gr_rds_make_data_decoder ();

/*!
 * \brief **FIXME**
 * \ingroup **FIXME**
 *
 * \sa **FIXME**
 */
class gr_rds_data_decoder : public gr_sync_block
{
private:

  enum state_t { ST_NO_SYNC, ST_SYNC };		// States


  int		d_collecting;			// NUmber of groups collected for syncing
  int 		COLLECT_GROUPS_FOR_SYNC;
  
  state_t	d_state;			// State holder

  bool 		d_sync[104];			// Containing 104 (a Group) bits
  bool	 	d_and[104];
  bool		d_nor[104];
  
  // State changing funcs
  void	enter_no_sync();
  void  enter_sync();

  void  reset_all_RDS_data();


  // RDS related methods / functions 
  //
  enum offset_word_t { A=0, B=1, C=2, C_=3, D=4, E=5 };
  bool	check_data(const bool *block, offset_word_t  offset);	// Calculate checksum
  void	decode_PI(const bool *block);
  int	decode_grouptype(const bool *block);
  bool  is_grouptype_A (const bool* block);
  void  decode_TP_PTY(const bool* block);
  void 	decode_PS_DI(const bool* block);
  void  decode_AF(const bool* block);
  void  decode_RT(const bool* block);

  char  transform_char(char z);
  
  // RDS related data
  unsigned int	d_PI_CI;		// PI Country Identification
  unsigned int  d_PI_AC;		// PI Area Coverage
  unsigned int  d_PI_PRN;		// PI Programme Reference Number
  bool		d_TP;			// Traffic Programme
  bool		d_TA;			// Traffic Announcment
  unsigned int  d_PTY;			// Programme TYpe
  char		d_PS[8];		// Programme Service Name
  bool		d_DI_Stereo;		// DI Mono=false,  Stereo=true
  bool		d_DI_AH;		// DI Not Artifical Head=false
  bool		d_DI_Compressed;	// DI Not compressed = false
  bool		d_DI_dyn_PTY;		// DI Static PTY = false
//  std::vector<double> d_AF;			// Alternative Frequencies
  
  //std::vector<string> d_RT;		// Radio Text feature
  char		d_RT[2][64];		// Radio Text a/b
  
  bool	grouptypes[2][16];		// Collect which grouptypes present
  int	d_invalid_block_counter;


public:
  int	get_state(void) { return d_state; }
  unsigned int get_PI_CI(void) { return d_PI_CI; }
  unsigned int get_PI_AC(void) { return d_PI_AC; }
  unsigned int get_PI_PRN(void) { return d_PI_PRN; }
  bool	get_TP(void) { return d_TP; }
  bool  get_TA(void) { return d_TA; }
  unsigned int get_PTY(void) { return d_PTY; }
  bool 	get_DI_Stereo(void) { return d_DI_Stereo; }
  bool	get_DI_AH(void)	{ return d_DI_AH; }
  bool	get_DI_Compressed(void) { return d_DI_Compressed; }
  bool 	get_DI_dyn_PTY(void) { return d_DI_dyn_PTY; }
//  string get_PS(void) { string s = string('c'); return s; }

private:
  // The friend declaration allows howto_make_square_ff to
  // access the private constructor.

  friend gr_rds_data_decoder_sptr gr_rds_make_data_decoder ();

  gr_rds_data_decoder ();  	// private constructor

 public:
  ~gr_rds_data_decoder ();	// public destructor

  // Where all the action really happens

  int work (int noutput_items,
		    gr_vector_const_void_star &input_items,
		    gr_vector_void_star &output_items);
};

#endif /* INCLUDED_gr_rds_data_decoder_H */
