#!/usr/bin/env python
#
# Copyright 2004 Free Software Foundation, Inc.
# 
# This file is part of GNU Radio
# 
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
# 

from gnuradio import gr
from gnuradio import mc4020
from gnuradio import blks
from gnuradio.eng_option import eng_option
from gnuradio.wxgui import stdgui, fftsink, scopesink
from optparse import OptionParser
import rds_rx
import wx
import sys
    
class app_flow_graph (stdgui.gui_flow_graph):
    def __init__(self, frame, panel, vbox, argv):
        stdgui.gui_flow_graph.__init__ (self, frame, panel, vbox, argv)
        
        parser = OptionParser (option_class=eng_option)
        (options, args) = parser.parse_args ()

   	input_rate = 20e6
   	cfir_decimation = 100                       # 200 hz

    	quad_rate = input_rate / cfir_decimation

    	# use high speed ADC as input source
    	src = mc4020.source (input_rate, mc4020.MCC_CH3_EN | mc4020.MCC_ALL_1V)

    	# compute FIR filter taps for channel selection
    	channel_coeffs = \
      	 gr.firdes.low_pass (2,          # gain
                          input_rate,   # sampling rate
                          250e3,        # low pass cutoff freq
                          80e3,      # width of trans. band
                          gr.firdes.WIN_HAMMING)

    	# input: short; output: complex
    	chan_filter1 = \
     		 gr.freq_xlating_fir_filter_scf (cfir_decimation,
                                      channel_coeffs,
                                      5.75e6,         # IF of tuner
                                      input_rate)

    	fm_demod_gain = 3200.0/32768.0

    	volume = 100.0

    	# input: complex; output: float
    	fm_demod = gr.quadrature_demod_cf (volume*fm_demod_gain)

    	# compute FIR filter taps for audio filter
    	width_of_transition_band = 10e3
    	coeffs = gr.firdes.low_pass (100.0,            # gain
                                 quad_rate,      # sampling rate
                                 70e3,
                                 width_of_transition_band,
                                 gr.firdes.WIN_HAMMING)

    	# input: float; output: float
    	fm_filter = gr.fir_filter_fff (1, coeffs)

    	self.connect(src, chan_filter1)
    	self.connect(chan_filter1, fm_demod)
    	self.connect(fm_demod, fm_filter)

	
        scope = scopesink.scope_sink_f (fg=self, parent=panel, title="SCOPE", sample_rate=quad_rate)
	fft_sink2, fft_win = fftsink.make_fft_sink_f(self, panel, "FFT", 512, quad_rate,  -20, 40)


	vbox.Add (fft_win, 1, wx.EXPAND)

	vbox.Add (scope.win, 1, wx.EXPAND)
	
	rds_receiver = rds_rx.rds_rx(self, quad_rate)

	self.connect(fm_filter, rds_receiver)

	self.connect(fm_filter,  fft_sink2)

	
	self.connect(rds_receiver.rds_bb_filter, scope)	


def main (args):

    nargs = len(args)
    if nargs == 1:
    	freq = float (args[0]) *1e6
    else:
    	print "usage: rds_demo_app.py freq_to_tune"

    print "Tuning freq"
    print freq
    print "\n"

    rf_front_end = gr.microtune_4937_eval_board ()
    if not rf_front_end.board_present_p ():
    	raise IOError, 'RF front end not found'

    rf_front_end.set_AGC(500)
    rf_front_end.set_RF_freq(freq)
    
    app = stdgui.stdapp (app_flow_graph, "RDS Receiver")
    app.MainLoop ()

if __name__ == '__main__':
    main (sys.argv[1:])
